import "../chunk-G2ADBYYC.js";
import { on, off, addClass, removeClass } from "../common/deps/dom";
import { emitEvent } from "../common/event";
import { getDomNode } from "../common/deps/dom";
const computedAnimationName = ({ constants, props }) => () => props.rightSlide ? constants.DIALOG_SLIDER_RIGHT : constants.DIALOG_FADE;
const computedAddUnit = (value) => isNaN(Number(value)) ? value : value + "px";
const computedStyle = ({ props, state }) => () => {
  const style = {};
  let { width, top, rightSlide, maxHeight } = props;
  if (top === void 0) {
    top = rightSlide ? "0" : "15vh";
  }
  width = computedAddUnit(width);
  top = computedAddUnit(top);
  maxHeight = computedAddUnit(maxHeight);
  if (!state.isFull) {
    style.width = width;
    style.top = state.top || top;
    style.maxHeight = maxHeight;
    if (rightSlide) {
      style.right = 0;
      style.height = "calc(100vh - " + style.top + ")";
    } else {
      style.left = state.left || "calc((100vw - " + width + ") / 2)";
    }
  }
  return style;
};
const computedBodyStyle = ({ props }) => () => {
  const style = {
    maxHeight: ""
  };
  let { maxHeight } = props;
  if (maxHeight) {
    style.maxHeight = "none";
  }
  return style;
};
const watchVisible = ({
  api,
  constants,
  emit,
  nextTick,
  parent,
  props,
  vm,
  state
}) => (val) => {
  const el = parent.$el;
  if (props.lockScroll) {
    val ? api.showScrollbar() : api.hideScrollbar();
  }
  state.move = false;
  state.isFull = props.fullscreen;
  emit("update:visible", val);
  if (val) {
    state.closed = false;
    emit("open");
    on(el, "scroll", api.updatePopper);
    nextTick(() => {
      vm.$refs.dialog.scrollTop = 0;
    });
    if (props.appendToBody) {
      document.body.appendChild(el);
    }
  } else {
    off(el, "scroll", api.updatePopper);
    if (!state.closed) {
      state.emitter.emit("boxclose", props.isFormReset);
      emit("close");
    }
    if (props.destroyOnClose) {
      nextTick(() => state.key++);
    }
    if (props.rightSlide) {
      const dialogBoxDom = el.querySelector(constants.DIALOG_BOX_CLASS) || el;
      dialogBoxDom.style.left = "";
    }
  }
};
const mounted = ({ api, parent, props }) => () => {
  if (props.lockScroll && props.visible) {
    api.showScrollbar();
  }
  if (props.visible) {
    const el = parent.$el;
    api.open();
    if (props.appendToBody) {
      document.body.appendChild(el);
    }
  }
};
const unMounted = ({ api, parent, props }) => () => {
  const el = parent.$el;
  api.hideScrollbar();
  if (props.appendToBody && el && el.parentNode) {
    el.parentNode.removeChild(el);
  }
};
const handleWrapperClick = ({ api, props }) => () => {
  if (!props.closeOnClickModal) {
    return;
  }
  api.handleClose("mask");
};
const handleClose = ({
  api,
  constants,
  emit,
  parent,
  props
}) => (type = "close") => {
  if (typeof props.beforeClose === "function" && props.beforeClose(type) === false) {
    return;
  }
  const el = parent.$el;
  if (props.rightSlide) {
    const dialogBoxDom = el.querySelector(constants.DIALOG_BOX_CLASS) || el;
    dialogBoxDom.style.left = "";
  }
  if (!emitEvent(emit, "before-close", api.hide)) {
    return;
  }
  api.hide();
};
const hide = ({ api, emit, state, props }) => (cancel) => {
  if (cancel !== false) {
    state.emitter.emit("boxclose", props.isFormReset);
    emit("update:visible", false);
    emit("change", false);
    emit("close");
    state.closed = true;
    api.hideScrollbar();
  }
};
const handleConfirm = ({ api, emit }) => () => {
  emit("confirm");
  api.handleClose();
};
const handleCancel = ({ api, emit }) => () => {
  emit("cancel");
  api.handleClose();
};
const updatePopper = ({ api, constants }) => () => {
  api.broadcast(constants.SELECT_DROPDOWN, "updatePopper");
  api.broadcast(constants.DROPDOWN_MENU, "updatePopper");
};
const afterEnter = (emit) => () => {
  emit("opened");
};
const afterLeave = (emit) => () => {
  emit("closed");
};
const findPopoverComponent = ({
  vm,
  componentList
}) => {
  const children = vm.$children;
  if (!children || children.length === 0) {
    return [];
  }
  children.forEach((child) => {
    const tag = child.$options.componentName;
    if (tag === "Select") {
      componentList.push(child);
    }
    findPopoverComponent({ vm: child, componentList });
  });
  return componentList;
};
const closeAllPopover = (parent) => {
  findPopoverComponent({ vm: parent, componentList: [] }).forEach((component) => {
    component.state.visible = false;
  });
};
const handleDrag = ({ parent, props, state, emit }) => (event) => {
  if (!props.draggable) {
    return;
  }
  let modalBoxElem = parent.$el.querySelector(".tiny-dialog-box");
  event.preventDefault();
  let demMousemove = document.onmousemove;
  let demMouseup = document.onmouseup;
  let disX = event.clientX - modalBoxElem.offsetLeft;
  let disY = event.clientY - modalBoxElem.offsetTop;
  let { visibleHeight, visibleWidth } = getDomNode();
  document.onmousemove = (event2) => {
    event2.preventDefault();
    if (!state.move) {
      emit("drag-start", event2);
      closeAllPopover(parent);
      state.move = true;
    }
    let offsetWidth = modalBoxElem.offsetWidth;
    let offsetHeight = modalBoxElem.offsetHeight;
    let maxX = Math.max(visibleWidth - offsetWidth, 0);
    let maxY = Math.max(visibleHeight - offsetHeight, 0);
    let left = event2.clientX - disX;
    let top = event2.clientY - disY;
    left = left < 0 ? 0 : left > maxX ? maxX : left;
    top = top < 0 ? 0 : top > maxY ? maxY : top;
    modalBoxElem.style.left = `${left}px`;
    modalBoxElem.style.top = `${top}px`;
    state.left = `${left}px`;
    state.top = `${top}px`;
    state.emitter.emit("boxdrag");
    emit("drag-move", event2);
  };
  document.onmouseup = () => {
    document.onmousemove = demMousemove;
    document.onmouseup = demMouseup;
    state.move = false;
    props.draggable && emit("drag-end", event);
  };
};
const showScrollbar = (lockScrollClass) => () => {
  addClass(document.body, lockScrollClass);
};
const hideScrollbar = (lockScrollClass) => () => {
  removeClass(document.body, lockScrollClass);
};
export {
  afterEnter,
  afterLeave,
  computedAddUnit,
  computedAnimationName,
  computedBodyStyle,
  computedStyle,
  handleCancel,
  handleClose,
  handleConfirm,
  handleDrag,
  handleWrapperClick,
  hide,
  hideScrollbar,
  mounted,
  showScrollbar,
  unMounted,
  updatePopper,
  watchVisible
};
