import {
  __spreadProps,
  __spreadValues
} from "../chunk-G2ADBYYC.js";
import {
  blur,
  showBox,
  watchFormSelect,
  clear,
  focus,
  select,
  getInput,
  textLength,
  handleBlur,
  handleFocus,
  handleInput,
  handleChange,
  calcIconOffset,
  resizeTextarea,
  getSuffixVisible,
  updateIconOffset,
  calcTextareaHeight,
  setNativeInputValue,
  calculateNodeStyling,
  handleCompositionEnd,
  handlePasswordVisible,
  handleCompositionStart,
  handleCompositionUpdate,
  hasSelection,
  handleEnterDisplayOnlyContent,
  hiddenPassword,
  dispatchDisplayedValue,
  getDisplayedValue
} from "./index";
import useStorageBox from "../tall-storage/vue-storage-box";
const api = [
  "blur",
  "showBox",
  "clear",
  "focus",
  "state",
  "select",
  "getInput",
  "handleBlur",
  "handleInput",
  "handleFocus",
  "handleChange",
  "calcIconOffset",
  "resizeTextarea",
  "getSuffixVisible",
  "updateIconOffset",
  "calcTextareaHeight",
  "setNativeInputValue",
  "calculateNodeStyling",
  "handleCompositionEnd",
  "handlePasswordVisible",
  "handleCompositionStart",
  "handleCompositionUpdate",
  "addMemory",
  "searchMemory",
  "selectedMemory",
  "storageData",
  "isMemoryStorage",
  "hasSelection",
  "handleEnterDisplayOnlyContent",
  "hiddenPassword"
];
const initState = ({
  reactive,
  computed,
  mode,
  props,
  parent,
  constants,
  api: api2
}) => {
  const state = reactive({
    mode,
    focused: false,
    hovering: false,
    isComposing: false,
    passwordVisible: false,
    boxVisibility: false,
    textareaCalcStyle: {},
    checkedLable: "",
    sheetvalue: props.modelValue,
    inputSize: computed(() => props.size || state.formItemSize),
    showClear: computed(
      () => props.clearable && !state.inputDisabled && !props.readonly && state.nativeInputValue && (state.focused || state.hovering)
    ),
    upperLimit: computed(() => parent.$attrs.maxlength),
    textLength: computed(() => textLength(props.modelValue)),
    inputExceed: computed(() => state.isWordLimitVisible && state.textLength > state.upperLimit),
    formItemSize: computed(() => (parent.formItem || {}).formItemSize),
    validateIcon: computed(() => constants.VALIDATE_ICON[state.validateState]),
    showWordLimit: computed(() => props.showWordLimit && parent.$attrs.maxlength),
    inputDisabled: computed(
      () => props.disabled || (parent.tinyForm || {}).disabled || state.isDisplayOnly || (parent.tinyForm || {}).displayOnly
    ),
    validateState: computed(() => parent.formItem ? parent.formItem.validateState : ""),
    textareaStyle: computed(() => __spreadProps(__spreadValues({}, state.textareaCalcStyle), {
      resize: props.resize
    })),
    needStatusIcon: computed(() => parent.tinyForm ? parent.tinyForm.statusIcon : false),
    showPwdVisible: computed(
      () => props.showPassword && !state.inputDisabled && !props.readonly && (!!state.nativeInputValue || state.focused)
    ),
    nativeInputValue: computed(
      () => props.modelValue === null || props.modelValue === void 0 ? "" : String(props.modelValue)
    ),
    isWordLimitVisible: computed(
      () => (props.showWordLimit && parent.$attrs.maxlength || props.counter) && (parent.type === "text" || parent.type === "textarea") && !state.inputDisabled && !props.readonly && !props.showPassword
    ),
    isDisplayOnly: computed(
      () => (props.displayOnly || (parent.tinyForm || {}).displayOnly) && ["text", "textarea", "password", "number"].includes(props.type)
    ),
    displayOnlyTooltip: "",
    hiddenPassword: computed(() => api2.hiddenPassword())
  });
  return state;
};
const initApi = ({
  api: api2,
  state,
  dispatch,
  broadcast,
  emit,
  refs,
  props,
  CLASS_PREFIX,
  parent
}) => {
  Object.assign(api2, {
    state,
    dispatch,
    broadcast,
    showBox: showBox(state),
    clear: clear(emit),
    getInput: getInput(refs),
    handleChange: handleChange(emit),
    watchFormSelect: watchFormSelect({ emit, props, state }),
    calcIconOffset: calcIconOffset({ CLASS_PREFIX, parent }),
    getSuffixVisible: getSuffixVisible({ parent, props, state }),
    calculateNodeStyling: calculateNodeStyling(),
    handleCompositionStart: handleCompositionStart(state),
    handleCompositionUpdate: handleCompositionUpdate(state),
    dispatchDisplayedValue: dispatchDisplayedValue({ state, props, dispatch, api: api2 }),
    getDisplayedValue: getDisplayedValue({ state, props })
  });
};
const mergeApi = ({
  storages,
  api: api2,
  componentName,
  props,
  emit,
  eventName,
  nextTick,
  parent,
  state,
  refs
}) => {
  const { storageData, isMemoryStorage, addMemory, searchMemory, selectedMemory } = storages;
  return Object.assign(api2, {
    addMemory,
    storageData,
    searchMemory,
    selectedMemory,
    isMemoryStorage,
    blur: blur(api2),
    focus: focus(api2),
    select: select(api2),
    handleBlur: handleBlur({
      api: api2,
      componentName,
      emit,
      eventName: eventName.blur,
      props,
      state
    }),
    handleFocus: handleFocus({ api: api2, emit, state }),
    handleInput: handleInput({ api: api2, emit, nextTick, state }),
    resizeTextarea: resizeTextarea({ api: api2, parent, refs, state }),
    updateIconOffset: updateIconOffset(api2),
    calcTextareaHeight: calcTextareaHeight({
      api: api2,
      hiddenTextarea: null,
      props,
      state
    }),
    setNativeInputValue: setNativeInputValue({ api: api2, state }),
    handleCompositionEnd: handleCompositionEnd({ api: api2, state }),
    handlePasswordVisible: handlePasswordVisible({ api: api2, nextTick, state }),
    hasSelection: hasSelection(api2),
    handleEnterDisplayOnlyContent: handleEnterDisplayOnlyContent({ state, props }),
    hiddenPassword: hiddenPassword({ state, props })
  });
};
const initWatch = ({
  watch,
  state,
  api: api2,
  props,
  nextTick,
  emit,
  componentName,
  eventName
}) => {
  watch(
    () => props.modelValue,
    (value) => {
      if (state.mode === "mobile") {
        state.sheetvalue = value;
        emit("update:modelValue", value);
      }
      nextTick(api2.resizeTextarea);
      if (props.validateEvent) {
        api2.dispatch(componentName, eventName.change, [value]);
      }
    }
  );
  watch(
    () => props.size,
    () => nextTick(api2.resizeTextarea),
    { immediate: true }
  );
  watch(
    () => state.nativeInputValue,
    () => {
      api2.setNativeInputValue();
    }
  );
  watch(
    () => props.type,
    () => {
      nextTick(() => {
        api2.setNativeInputValue();
        api2.resizeTextarea();
        api2.updateIconOffset();
      });
    }
  );
  watch(
    () => state.isDisplayOnly,
    () => {
      nextTick(() => {
        api2.setNativeInputValue();
        api2.resizeTextarea();
        api2.updateIconOffset();
        api2.dispatchDisplayedValue();
      });
    }
  );
  watch(
    () => state.sheetvalue,
    (value) => api2.watchFormSelect(value),
    { immediate: true }
  );
};
const renderless = (props, { computed, onMounted, onUpdated, reactive, toRefs, watch, inject }, { vm, refs, parent, emit, constants, nextTick, broadcast, dispatch, mode }) => {
  const api2 = {};
  const componentName = constants.COMPONENT_NAME.FormItem;
  const eventName = { change: "form.change", blur: "form.blur" };
  const CLASS_PREFIX = {
    Input: constants.inputMode(mode),
    InputGroup: constants.inputGroupMode(mode)
  };
  const state = initState({ reactive, computed, mode, props, parent, constants, api: api2 });
  initApi({ api: api2, state, dispatch, broadcast, emit, refs, props, CLASS_PREFIX, parent });
  const storages = useStorageBox({ api: api2, props, reactive, toRefs });
  parent.tinyForm = parent.tinyForm || inject("form", null);
  mergeApi({ api: api2, storages, componentName, emit, eventName, props, state, nextTick, parent, refs });
  initWatch({ watch, state, api: api2, props, nextTick, emit, componentName, eventName });
  onMounted(() => {
    api2.setNativeInputValue();
    api2.resizeTextarea();
    api2.updateIconOffset();
    api2.dispatchDisplayedValue();
    dispatch("Select", "input-mounted", vm.$el);
    dispatch("Tooltip", "tooltip-update", vm.$el);
  });
  onUpdated(() => {
    nextTick(api2.updateIconOffset);
  });
  return api2;
};
export {
  api,
  renderless
};
