import {
  __spreadValues
} from "../chunk-G2ADBYYC.js";
import { on, off } from "../common/deps/dom";
const initPullRefresh = ({ t, props, state }) => () => {
  var _a, _b, _c;
  const defaultOption = {
    pullingUpText: t("ui.pullRefresh.pullingUp"),
    pullingDownText: t("ui.pullRefresh.pullingDown"),
    pullUpDisabled: false,
    pullDownDisabled: false,
    headHeight: 50,
    footHeight: 50
  };
  state.pullUp = __spreadValues(__spreadValues({}, defaultOption), props.pullUp);
  state.pullDown = __spreadValues(__spreadValues({}, defaultOption), props.pullDown);
  state.refreshStyle = {
    paddingTop: state.pullDown.headHeight + "px",
    paddingBottom: state.pullUp.footHeight + "px"
  };
  state.loosingText = (_a = props.loosingText) != null ? _a : t("ui.pullRefresh.loosing");
  state.successText = (_b = props.successText) != null ? _b : t("ui.pullRefresh.success");
  state.failedText = (_c = props.failedText) != null ? _c : t("ui.pullRefresh.failed");
};
const onTouchstart = (state) => (event) => {
  state.draggposition = event.touches[0].clientY;
};
const onTouchmove = ({ props, state }) => (event) => {
  if (event.touches[0].clientY < state.draggposition) {
    if (!state.pullUp.pullUpDisabled) {
      if (props.hasMore) {
        state.translate3d = (event.touches[0].clientY - state.draggposition) / 2;
        state.pullUpReplaces = Math.abs(state.translate3d) > state.pullUp.footHeight ? state.loosingText : state.pullUp.pullingUpText;
        state.pullDownReplaces = "";
      }
    }
  } else {
    if (!state.pullDown.pullDownDisabled) {
      state.translate3d = (event.touches[0].clientY - state.draggposition) / 2;
      state.pullDownReplaces = Math.abs(state.translate3d) > state.pullDown.headHeight ? state.loosingText : state.pullDown.pullingDownText;
      state.pullUpReplaces = "";
    }
  }
  state.animationDuration = 0;
  state.pullUpLoading = false;
  state.pullDownLoading = false;
};
const onTouchend = ({ api, props, state }) => () => {
  state.animationDuration = props.animationDuration;
  if (Math.abs(state.translate3d) >= state.pullDown.headHeight || Math.abs(state.translate3d) >= state.pullUp.footHeight) {
    if (state.translate3d >= 0) {
      state.translate3d = state.pullDown.headHeight;
      const { handler } = state.pullDown;
      const pullDownPromise = handler && handler();
      state.pullDownLoading = true;
      if (pullDownPromise == null ? void 0 : pullDownPromise.then) {
        pullDownPromise.then(
          () => {
            api.handlerModelValue("down", "success");
          },
          (e) => {
            api.handlerModelValue("down", "failed");
          }
        );
        return;
      }
      console.error(new Error("handler down is not promise"));
    } else {
      state.translate3d = -state.pullUp.footHeight;
      const { handler } = state.pullUp;
      const pullUpPromise = handler && handler();
      state.pullUpLoading = true;
      if (pullUpPromise == null ? void 0 : pullUpPromise.then) {
        pullUpPromise.then(
          () => {
            api.handlerModelValue("up", "success");
          },
          (e) => {
            api.handlerModelValue("up", "failed");
          }
        );
        return;
      }
      console.error(new Error("handler up is not promise"));
    }
    api.clearPullRefresh();
  } else {
    state.pullUpLoading = false;
    state.pullDownLoading = false;
    api.clearPullRefresh();
  }
};
const mountedHandler = ({ api, refs }) => () => {
  const track = refs.track;
  on(track, "touchstart", api.onTouchstart);
  on(track, "touchmove", api.onTouchmove);
  on(track, "touchend", api.onTouchend);
};
const beforeUnmountHandler = ({ api, refs }) => () => {
  const track = refs.track;
  off(track, "touchstart", api.onTouchstart);
  off(track, "touchmove", api.onTouchmove);
  off(track, "touchend", api.onTouchend);
};
const handlerModelValue = ({ api, state }) => (value, result) => {
  state.pullUpLoading = false;
  state.pullDownLoading = false;
  if (value === "down") {
    state.pullDownReplaces = state[`${result}Text`];
  } else {
    state.pullUpReplaces = state[`${result}Text`];
  }
  setTimeout(() => {
    api.clearPullRefresh();
  }, state.successDuration);
};
const clearPullRefresh = (state) => () => {
  state.translate3d = 0;
  state.pullUpReplaces = "";
  state.pullDownReplaces = "";
};
export {
  beforeUnmountHandler,
  clearPullRefresh,
  handlerModelValue,
  initPullRefresh,
  mountedHandler,
  onTouchend,
  onTouchmove,
  onTouchstart
};
