import {
  __spreadProps,
  __spreadValues
} from "../chunk-G2ADBYYC.js";
import {
  handleChange,
  setLink,
  handleMove,
  handleClickOutside,
  removeClickOutside,
  handleClick,
  shouldShow,
  eventImg,
  eventClick,
  Active
} from "./index";
const api = [
  "toolBar",
  "state",
  "setLink",
  "handleChange",
  "box",
  "handleMove",
  "handleClickOutside",
  "removeClickOutside",
  "handleClick",
  "shouldShow",
  "fontSize",
  "eventImg",
  "eventClick",
  "Active"
];
const renderless = (props, { computed, onMounted, onBeforeUnmount, reactive, ref }, { vm, emit, parent }, {
  Editor,
  Collaboration,
  CodehighComp,
  Y,
  WebrtcProvider,
  StarterKit,
  Table,
  TableCell,
  TableHeader,
  TableRow,
  Color,
  TextStyle,
  Image,
  Highlight,
  Link,
  Underline,
  Subscript,
  Superscript,
  TaskItem,
  TaskList,
  TextAlign,
  Paragraph,
  mergeAttributes,
  CodeBlockLowlight,
  lowlight,
  VueNodeViewRenderer,
  NodeViewContent,
  nodeViewProps,
  NodeViewWrapper,
  Placeholder,
  codeHighlight
}) => {
  var _a;
  let toolBar = [
    "bold",
    "italic",
    "underline",
    "strike",
    "quote",
    "code",
    "codeBlock",
    "unorderedlist",
    "orderedlist",
    "taskList",
    "subscript",
    "superscript",
    // 'nodeDelete',
    "undo",
    "redo",
    "left",
    "center",
    "right",
    "h-box",
    "font-size",
    "line-height",
    "highlight",
    "color",
    "backgroundColor",
    "formatClear",
    "link",
    "unlink",
    "img",
    "table"
  ];
  if (props.customToolBar) {
    toolBar = props.customToolBar;
  }
  let ydoc = null;
  let provider = null;
  const CustomImage = Image.extend({
    addAttributes() {
      var _a2;
      return __spreadProps(__spreadValues({}, (_a2 = this.parent) == null ? void 0 : _a2.call(this)), {
        type: {
          default: "img"
        }
      });
    },
    renderHTML({ node, HTMLAttributes }) {
      if (node.attrs.type === "video") {
        return ["div", { class: "img-button" }, ["video", mergeAttributes({ controls: true }, HTMLAttributes)]];
      } else {
        return ["div", { class: "img-button" }, ["img", HTMLAttributes]];
      }
    }
  });
  const CustomParagraph = Paragraph.extend({
    addOptions() {
      return {
        levels: [1, 1.5, 2, 2.5, 3]
      };
    },
    addAttributes() {
      return {
        level: {
          default: 1
        }
      };
    },
    renderHTML({ node, HTMLAttributes }) {
      const hasLevel = this.options.levels.includes(node.attrs.level);
      const level = hasLevel ? node.attrs.level : this.options.levels[0];
      return ["p", mergeAttributes({ style: `line-height: ${level}` }, HTMLAttributes), 0];
    },
    addCommands() {
      return {
        setP: (attributes) => ({ commands }) => {
          return commands.setNode(this.name, attributes);
        }
      };
    }
  });
  const CustomSize = Paragraph.extend({
    addOptions() {
      return {
        size: [12, 14, 16, 18, 20, 24, 30]
      };
    },
    addAttributes() {
      return {
        size: {
          default: 16
        }
      };
    },
    renderHTML({ node, HTMLAttributes }) {
      const hasSize = this.options.size.includes(node.attrs.size);
      const size = hasSize ? node.attrs.size : this.options.size[2];
      return ["p", mergeAttributes({ style: `font-size: ${size}px` }, HTMLAttributes), 0];
    },
    addCommands() {
      return {
        setSize: (attributes) => ({ commands }) => {
          return commands.setNode(this.name, attributes);
        }
      };
    }
  });
  const CustomBackgroundColor = Highlight.extend({
    addAttributes() {
      return {
        bgColor: {
          default: null,
          renderHTML: (attributes) => {
            if (!attributes.bgColor) {
              return {};
            }
            return {
              style: `background: ${attributes.bgColor}`
            };
          }
        }
      };
    },
    addCommands() {
      return {
        setBackColor: (attributes) => ({ commands }) => {
          return commands.setMark(this.name, attributes);
        }
      };
    }
  });
  const defaultOptions = __spreadValues({
    extensions: [
      StarterKit == null ? void 0 : StarterKit.configure({
        // 开启多人协作功能要关闭默认的history模式
        history: false
      }),
      Table.configure({
        resizable: true
      }),
      TableCell,
      TableHeader,
      TableRow,
      Color,
      TextStyle,
      CustomImage,
      Highlight,
      Link,
      Underline,
      Subscript,
      Superscript,
      TaskList,
      TaskItem.configure({
        nested: true
      }),
      TextAlign.configure({
        types: ["heading", "paragraph"]
      }),
      CustomParagraph,
      CustomSize,
      CustomBackgroundColor,
      CodeBlockLowlight.extend({
        addNodeView() {
          return VueNodeViewRenderer(codeHighlight);
        }
      }).configure({ lowlight }),
      Placeholder.configure({
        placeholder: (_a = props.placeholder) != null ? _a : "Write something \u2026"
      })
    ],
    content: "",
    autofocus: true,
    editable: true,
    injectCSS: false,
    // 事件
    onBeforeCreate({ editor: editor2 }) {
      emit("beforeCreate", { editor: editor2 });
    },
    onCreate({ editor: editor2 }) {
      emit("create", { editor: editor2 });
    },
    onUpdate({ editor: editor2 }) {
      const json = editor2.getJSON();
      const html = editor2.getHTML();
      const text = editor2.getText();
      const lineText = editor2.getText({ blockSeparator: "--" });
      emit("update", { editor: editor2 });
      emit("update:modelValue", html);
    },
    onFocus({ editor: editor2, event }) {
      emit("focus", { editor: editor2, event });
    },
    onBlur({ editor: editor2, event }) {
      emit("blur", { editor: editor2, event });
    },
    onSelectionUpdate({ editor: editor2 }) {
      emit("selectionUpdate", { editor: editor2 });
    },
    onTransaction({ editor: editor2, transaction }) {
      emit("transaction", { editor: editor2, transaction });
    },
    onDestroy() {
      emit("destroy");
    }
  }, props.options);
  if (props.Collaboration) {
    if (!window._yDoc) {
      window._yDoc = new Y.Doc();
    }
    ydoc = window._yDoc;
    provider = new WebrtcProvider("tiny-examsple-document", ydoc);
    defaultOptions.extensions.push(
      Collaboration == null ? void 0 : Collaboration.configure({
        document: ydoc
      })
    );
  }
  const editor = new Editor(props.options ? props.options : defaultOptions);
  const box = ref(null);
  const fontSize = ref("16px");
  const state = reactive({
    editor: null,
    // table 变量
    isShow: false,
    flagX: 0,
    flagY: 0
  });
  state.editor = editor;
  const api2 = {
    toolBar,
    state,
    setLink: setLink(editor),
    handleChange: handleChange(editor),
    // table处理函数
    box,
    handleMove: handleMove(state, box),
    handleClickOutside: handleClickOutside(state, box),
    removeClickOutside: removeClickOutside(state, box),
    handleClick: handleClick(state, box),
    // bubble 菜单
    shouldShow,
    //
    fontSize,
    eventImg,
    eventClick,
    Active
  };
  onBeforeUnmount(() => {
    state.editor.destroy();
  });
  return api2;
};
export {
  api,
  renderless
};
