import "../chunk-G2ADBYYC.js";
import { format } from "../common/date";
const getDate = (dateTime) => ({
  date: format(dateTime, "yyyy-MM-dd"),
  time: format(dateTime, "hh:mm")
});
const getStatus = ({ state, t }) => (value) => {
  const status = state.current - value;
  return status > 0 ? t("ui.steps.done") : status === 0 ? t("ui.steps.doing") : t("ui.steps.wait");
};
const computedWidth = () => (width) => {
  return /^\d+$/.test(width) ? `${width}px` : width;
};
const handleClick = ({ emit, state }) => (node) => {
  const { index } = node || {};
  if (!node.disabled) {
    emit("click", state.isReverse ? state.nodesLength - index - 1 : index, node);
  }
};
const getStatusCls = ({ constants, state }) => (node) => {
  const { index } = node;
  const { PROCESS_DONE_CLS, PROCESS_CUR_CLS, PROCESS_WAIT_CLS, PROCESS_DISABLED_CLS, PROCESS_ERROR_CLS } = constants;
  const cls = {};
  const reverse = state.isReverse;
  if (node == null ? void 0 : node.disabled) {
    cls[PROCESS_DISABLED_CLS] = true;
  } else if (node == null ? void 0 : node.error) {
    cls[PROCESS_ERROR_CLS] = true;
  } else {
    cls[PROCESS_DONE_CLS] = reverse ? index > state.current : index < state.current;
    cls[PROCESS_CUR_CLS] = index === state.current;
    cls[PROCESS_WAIT_CLS] = reverse ? index < state.current : index > state.current;
  }
  return cls;
};
const computedCurrent = ({ state, api }) => () => state.isReverse ? state.nodesLength - api.rootProps.active - 1 : api.rootProps.active;
const computedIsReverse = (api) => () => api.rootProps.reverse && api.rootProps.vertical;
const computedItemCls = ({ props, api, state }) => () => {
  const itemClass = [];
  if (api.rootProps.vertical) {
    itemClass.push("timeline");
  } else {
    itemClass.push("normal");
  }
  itemClass.push(api.getStatusCls(props.node));
  if (state.computedLineWidth) {
    itemClass.push("no-flex");
  }
  return itemClass;
};
const computedItemStyle = ({ props, state, api }) => () => {
  const { index } = props.node;
  const { computedSpace, nodesLength } = state;
  const { textPosition, vertical } = api.rootProps;
  if (vertical) {
    return { height: index === nodesLength - 1 ? "" : computedSpace || "88px" };
  }
  if (textPosition === "right") {
    if (computedSpace && !state.computedLineWidth) {
      return { flex: `0 0 ${computedSpace}` };
    }
    return null;
  }
  return { width: computedSpace || 100 / nodesLength + "%" };
};
export {
  computedCurrent,
  computedIsReverse,
  computedItemCls,
  computedItemStyle,
  computedWidth,
  getDate,
  getStatus,
  getStatusCls,
  handleClick
};
